# Generated by roxygen2: do not edit by hand

S3method(knit_print,grouped_df)
S3method(knit_print,output_format_dependency)
S3method(knit_print,rowwise_df)
S3method(knit_print,tbl_sql)
S3method(prepare_evaluate_output,default)
S3method(prepare_evaluate_output,htmlwidget)
S3method(prepare_evaluate_output,knit_asis)
S3method(prepare_evaluate_output,list)
S3method(print,paged_df)
export(all_output_formats)
export(available_templates)
export(beamer_presentation)
export(clean_site)
export(context_document)
export(convert_ipynb)
export(default_output_format)
export(default_site_generator)
export(draft)
export(find_external_resources)
export(find_pandoc)
export(from_rmarkdown)
export(github_document)
export(html_dependency_bootstrap)
export(html_dependency_codefolding_lua)
export(html_dependency_font_awesome)
export(html_dependency_highlightjs)
export(html_dependency_ionicons)
export(html_dependency_jquery)
export(html_dependency_jqueryui)
export(html_dependency_pagedtable)
export(html_dependency_tabset)
export(html_dependency_tocify)
export(html_document)
export(html_document_base)
export(html_fragment)
export(html_notebook)
export(html_notebook_metadata)
export(html_notebook_output_code)
export(html_notebook_output_html)
export(html_notebook_output_img)
export(html_notebook_output_png)
export(html_vignette)
export(includes)
export(includes_to_pandoc_args)
export(ioslides_presentation)
export(knit_params_ask)
export(knitr_options)
export(knitr_options_html)
export(knitr_options_pdf)
export(latex_dependency)
export(latex_dependency_tikz)
export(latex_document)
export(latex_fragment)
export(md_document)
export(metadata)
export(navbar_html)
export(navbar_links_html)
export(odt_document)
export(output_format)
export(output_format_dependency)
export(output_metadata)
export(paged_table)
export(pandoc_available)
export(pandoc_citeproc_args)
export(pandoc_citeproc_convert)
export(pandoc_convert)
export(pandoc_exec)
export(pandoc_highlight_args)
export(pandoc_include_args)
export(pandoc_latex_engine_args)
export(pandoc_lua_filter_args)
export(pandoc_metadata_arg)
export(pandoc_metadata_file_arg)
export(pandoc_options)
export(pandoc_path_arg)
export(pandoc_self_contained_html)
export(pandoc_template)
export(pandoc_toc_args)
export(pandoc_variable_arg)
export(pandoc_version)
export(parse_html_notebook)
export(pdf_document)
export(pkg_file_lua)
export(powerpoint_presentation)
export(publish_site)
export(relative_to)
export(render)
export(render_delayed)
export(render_site)
export(render_supporting_files)
export(resolve_output_format)
export(rmarkdown_format)
export(rtf_document)
export(run)
export(shiny_prerendered_chunk)
export(shiny_prerendered_clean)
export(shiny_prerendered_server_start_code)
export(site_config)
export(site_generator)
export(site_resources)
export(slidy_presentation)
export(word_document)
export(yaml_front_matter)
import(methods)
import(stats)
import(utils)
importFrom(evaluate,evaluate)
importFrom(htmltools,HTML)
importFrom(htmltools,copyDependencyToDir)
importFrom(htmltools,extractPreserveChunks)
importFrom(htmltools,htmlDependency)
importFrom(htmltools,makeDependencyRelative)
importFrom(htmltools,renderDependencies)
importFrom(htmltools,restorePreserveChunks)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(knitr,knit_print)
