#'---
#' title: "Page D"
#' output: html_document
#'---

knitr::opts_chunk$set(echo = TRUE)

#'
#' ## Section 1
#'
#' Lorem ipsum dolor sit amet, consectetur adipiscing elit. Quisque sapien nisl, egestas ut erat eu, egestas porttitor tellus. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Sed pellentesque volutpat justo, et cursus lorem pulvinar luctus. Curabitur porta, diam a pretium egestas, dui dolor tempus sem, non facilisis turpis tortor a lectus. Morbi nec enim aliquam, eleifend libero eget, efficitur quam. Fusce dapibus elit vitae tellus hendrerit tristique. Donec felis elit, gravida non orci consectetur, finibus sollicitudin justo. Curabitur imperdiet pellentesque elit ac ultricies. Vestibulum erat leo, tristique non felis a, fringilla pellentesque dolor. Vivamus sem justo, commodo sit amet ultrices in, lobortis sit amet lacus. Donec feugiat sem vel arcu feugiat, sed maximus elit aliquam. Pellentesque justo leo, pellentesque a mollis eget, suscipit ac turpis. Duis ornare dui quis nisi dictum, at consequat justo auctor.
#'
plot(cars)
#' ## Section 2
#'
#' Nullam ullamcorper ac nunc sed tempus. Suspendisse sagittis tortor auctor magna aliquam, eu porta libero pretium. Vestibulum placerat feugiat mattis. Nulla ornare luctus quam vel iaculis. Nulla ornare justo venenatis viverra mollis. Aliquam pretium dui sed sem accumsan blandit. Integer aliquam posuere odio, ac interdum lorem egestas in. Vivamus sed est dolor. Nulla cursus blandit elit nec porta. Etiam dictum libero vel magna pulvinar convallis. Proin lacinia elementum ligula a malesuada. Ut rhoncus tortor vitae lorem ultrices luctus. Aenean sit amet libero non metus fringilla lacinia id quis massa. Nulla sollicitudin tortor non turpis suscipit, in imperdiet ex finibus.
#'
#' ## Section 3
#'
#' Sed leo tortor, sagittis sit amet lacus eu, tincidunt ornare lacus. Integer lorem lacus, facilisis nec turpis et, fermentum scelerisque felis. Pellentesque ultrices nibh lacus, ut tincidunt metus finibus quis. Nunc tincidunt eget urna a lacinia. Sed auctor lorem eu ultrices interdum. Aliquam vehicula ornare massa, ut placerat mi imperdiet et. Sed facilisis eleifend tortor, id semper nisl congue eu. Donec malesuada dolor eget ex rutrum, nec consequat erat ultrices. Maecenas iaculis leo id est lacinia laoreet. Donec tempor cursus orci. Suspendisse potenti. Etiam cursus iaculis tincidunt.
#'
