% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/html_notebook.R
\name{parse_html_notebook}
\alias{parse_html_notebook}
\title{Parse an HTML Notebook}
\usage{
parse_html_notebook(path)
}
\arguments{
\item{path}{The path to an R Notebook file (with extension \code{.nb.html}).}
}
\description{
Parse an HTML notebook, retrieving annotation information
related to generated outputs in the document, as well as the
original R Markdown source document.
}
\details{
See the \href{https://rmarkdown.rstudio.com/r_notebook_format.html}{online
documentation} for additional details on using the \code{html_notebook}
format.
}
